/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.taskmapping.instantiate;

import java.util.ArrayList;
import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ComposableDecoderBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.ArchExpSubDecoder;
import org.fortiss.af3.exploration.moea.model.DseSpecification;
import org.opt4j.core.problem.Decoder;

import com.google.inject.Inject;

/**
 * Base class for {@link Decoder}s that instantiate {@link AbstractTaskMappingEncoding}s to
 * {@link InstantiatedTaskMappingEncoding}.
 * 
 * @author diewald
 */
public abstract class AbstractTaskMappingDecoder<S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
		extends ComposableDecoderBase<AbstractTaskMappingEncoding, T> implements ArchExpSubDecoder {
	/** Reference to the {@link DseSpecification} specification that defines the DSE. */
	DseSpecification explorationDefinition;

	/** Constructs a new decoder that instantiates {@link AbstractTaskMappingEncoding}. */
	@Inject
	public AbstractTaskMappingDecoder(DseSpecification explorationDefinition) {
		this.explorationDefinition = explorationDefinition;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> reqTypes = new ArrayList<>();
		reqTypes.add(AbstractTaskMappingEncoding.class);
		reqTypes.add(AbstractTaskGraphEncoding.class);
		reqTypes.add(PartitionMappingEncoding.class);

		return reqTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		Collection<Class<?>> optTypes = new ArrayList<>();
		optTypes.add(AbstractTaskGraphEncoding.class);
		return optTypes;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Class<InstantiatedTaskMappingEncoding> getOutputType() {
		return InstantiatedTaskMappingEncoding.class;
	}
}
