/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import static java.lang.Math.pow;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationTargetEvaluator;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.NoEmptyPartitionConstraint;
import org.fortiss.tooling.common.util.LambdaUtils;
import org.opt4j.core.DoubleValue;
import org.opt4j.core.Value;

import com.google.common.collect.ClassToInstanceMap;

/**
 * Adds a penalty for candidate solutions that contain empty partitions.
 * 
 * @author diewald
 */
public class NoEmptyPartitionConstraintEvaluator extends
		ConstraintEvaluatorBase<TaskMappingEncoding<?, ?>, Double, NoEmptyPartitionConstraint> {
	/** {@inheritDoc} */
	@Override
	public Value<?> evaluateGoal(TaskMappingEncoding<?, ?> phenotype,
			FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
			ClassToInstanceMap<EObject> transformedModels) throws Exception {
		PartitionMappingEncoding virtMapping =
				phenotypeTypeMap.getEncoding(PartitionMappingEncoding.class);
		assert (virtMapping != null);

		Collection<Partition> emptyPartitions =
				LambdaUtils.filter(virtMapping.getRequesters(), res -> phenotype.getRequesters(res)
						.isEmpty());
		return new DoubleValue(pow(10 * emptyPartitions.size(), 2));
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredInputTypes() {
		Collection<Class<?>> requiredTypes = new ArrayList<>();
		requiredTypes.add(TaskMappingEncoding.class);
		requiredTypes.add(PartitionMappingEncoding.class);
		return requiredTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<?> getPrimaryInputType() {
		return TaskMappingEncoding.class;
	}

	/** {@inheritDoc} */
	@Override
	public boolean validateInputs() throws Exception {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public boolean validateInternal(ExplorationConstraint<?> expTarget,
			TaskMappingEncoding<?, ?> solution, Value<?> value) {
		return value.getDouble() <= 10e-6;
	}

	/** {@inheritDoc} */
	@Override
	public
			IExplorationTargetEvaluator<TaskMappingEncoding<?, ?>, Double, NoEmptyPartitionConstraint>
			createEvaluator(SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter,
					NoEmptyPartitionConstraint explorationTarget) throws Exception {
		return new NoEmptyPartitionConstraintEvaluator();
	}
}
