/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.objective;

import java.math.BigInteger;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.evaluator.TaskMappingEvaluator;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationTargetEvaluator;
import org.fortiss.af3.exploration.dsl_v2.model.arithmetic.IArithmeticExpression;
import org.fortiss.af3.exploration.dsl_v2.model.function.Maximize;
import org.fortiss.af3.exploration.dsl_v2.model.function.Minimize;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.DoubleValue;
import org.opt4j.core.IntegerValue;
import org.opt4j.core.Value;
import org.opt4j.core.problem.Evaluator;

import com.google.common.collect.ClassToInstanceMap;

/**
 * 
 * @author diewald
 */
public class MappingEvaluatorObjective extends TaskMappingEvaluator {

	/** Prototype Constructor. */
	public MappingEvaluatorObjective() {
	}

	/**
	 * Constructor.
	 * 
	 * @param systemModelAdapter
	 *            abstracts the input models.
	 * @param explorationTarget
	 *            Target for which the {@link Evaluator} is constructed.
	 * @throws Exception
	 *             if the inputs don't satisfy the {@link Evaluator}'s requirements.
	 */
	public MappingEvaluatorObjective(SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter,
			ExplorationTarget<Double> explorationTarget) throws Exception {
		super(systemModelAdapter, explorationTarget);
	}

	/** {@inheritDoc} */
	@Override
	public
			Value<?>
			evaluateGoal(
					TaskMappingEncoding<ITaskAdapter<?>, TaskMappingEntry<ITaskAdapter<?>, Partition>> phenotype,
					FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
					ClassToInstanceMap<EObject> transformedModels) throws Exception {
		mapping = phenotype;
		virtMapping = phenotypeTypeMap.getEncoding(PartitionMappingEncoding.class);
		assert (virtMapping != null);

		// For ExplorationObjectives the expression begins either with a Minimize or Maximize
		// operation.
		// FIXME: bad dsl design...
		IArithmeticExpression minMaxTerm = null;
		if(expression instanceof Minimize) {
			minMaxTerm = ((Minimize)expression).getArgs();
		} else if(expression instanceof Maximize) {
			minMaxTerm = ((Maximize)expression).getArgs();
		}
		assert (minMaxTerm != null);

		// The minMax Term is always non-zero.
		// FIXME: Using "BigInteger" as a type is a hack: It is a workaround to comply with the
		// construction of objectives in the UI.
		@SuppressWarnings("null") Number retVal = minMaxTerm.accept(this, Number.class);

		return retVal instanceof BigInteger ? new IntegerValue(retVal.intValue())
				: new DoubleValue(retVal.doubleValue());
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Maximize max) throws Exception {
		// Max/Min operators are handled in the EvaluationWrapper (pre-processing).
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Minimize min) throws Exception {
		// Max/Min operators are handled in the EvaluationWrapper (pre-processing).
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public
			IExplorationTargetEvaluator<TaskMappingEncoding<ITaskAdapter<?>, TaskMappingEntry<ITaskAdapter<?>, Partition>>, Double, ExplorationTarget<Double>>
			createEvaluator(SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter,
					ExplorationTarget<Double> explorationTarget) throws InstantiationException,
					Exception {
		return new MappingEvaluatorObjective(systemModelAdapter, explorationTarget);
	}
}
