/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.faultmodel;

/**
 * Abstraction of {@link FaultDetector}s (providing information on the runtime overhead caused by
 * the {@link FaultDetector}, as well as its coverage (capability to detect faults).
 * 
 * @author huang
 */
public class FaultDetector implements java.lang.Comparable<FaultDetector> {

	/** {@link FaultDetector}'s runtime overhead. */
	private double overhead;

	/** {@link FaultDetector}'s coverage. */
	private double coverage;

	/** Constructs a new {@link FaultDetector}. */
	public FaultDetector(double o, double c) {
		overhead = o;
		coverage = c;
	}

	/** Returns the {@link FaultDetector}'s overhead. */
	public double getOverhead() {
		return overhead;
	}

	/** Returns the {@link FaultDetector}'s coverage. */
	public double getCoverage() {
		return coverage;
	}

	/** {@inheritDoc} */
	@Override
	public int compareTo(FaultDetector fd) {
		// sort the detector according to overhead
		if(overhead < fd.overhead) {
			return -1;
		} else if(overhead > fd.overhead) {
			return 1;
		} else {
			return(coverage <= fd.coverage ? -1 : 1);
		}
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {

		return "(" + overhead + "," + coverage + ")";
	}
}
