/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.sysmodel.mapping;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationEncoding;

/**
 * Generalized interface for accessing Request->Resource mappings that are the result of a DSE.
 * 
 * @author diewald
 */
public interface IMappingEncoding<S extends IRequestAdapter<?>, T extends IResourceAdapter<?>, M extends IMappingEntry<S, T>>
		extends IExplorationEncoding {

	/**
	 * Returns the allocation of a specific {@link IRequestAdapter} to its {@link IMappingEntry}s.
	 */
	public Collection<T> getAllocatedResourcesFor(S source);

	/**
	 * Returns the set of {@link IResourceAdapter}s to which {@link IRequestAdapter}s can be
	 * allocated.
	 */
	public Collection<T> getResources();

	/**
	 * Returns the collection of {@link IMappingEntry}s which represent the (multi-)allocation of
	 * the given {@link IRequestAdapter}s to {@link IResourceAdapter}s.
	 */
	public Collection<M> getMappingEntriesOf(S source);

	/**
	 * Returns whether {@code this} {@link IMappingEncoding} contains the given
	 * {@link IMappingEntry}.
	 */
	public boolean containsEntry(M entry);

	/**
	 * Returns the collection of {@link IMappingEntry}s contained in {@code this}
	 * {@link IMappingEncoding}.
	 */
	public Collection<M> getMappingEntries();

	/** Returns the collection of resource requesters. */
	public Collection<S> getRequesters();
}
